#DEFINE     end .end        ; This allows the use of TI's headers
#DEFINE     END .end
#DEFINE     equ .equ
#DEFINE     EQU .equ
#INCLUDE    "ti83asm.inc"   ; TI-83 header files
#INCLUDE    "tokens.inc"
.org        $9327           ; This is where we get relocated to

;               Logic Analyser
;
;               v1.3 for TI-83
;
;             By  Vasantha Crabb
;
; (C) 1999
;

;---------------------
; Program VARIABLES
;---------------------

MENU_ITEM   = STATVARS+$00          ; The setting being adjusted

ACQ_BUF     = STATVARS+$01          ; Acquisition buffer

;---------------------
; Program CONSTANTS
;---------------------

TRIG_MAN    = 1                     ; Manual trigger
TRIG_1R     = 2                     ; Channel 1 rising
TRIG_1F     = 3                     ; Channel 1 falling
TRIG_2R     = 4                     ; Channel 2 rising
TRIG_2F     = 5                     ; Channel 2 falling

Ch1Mask     = $04                   ; Channel 1 read bit
Ch1Bit      = 2
Ch2Mask     = $08                   ; Channel 2 read bit
Ch2Bit      = 3

;---------------------
; AShell program header
;---------------------

    nop                             ; This indicates that we're AShell compatible
    jr      prog_start              ; For fallback to ZASMLOAD
    .dw     $0000                   ; Version of header
    .dw     TitleStr                ; Points to the program description
    .dw     Icon                    ; Points to an 8x8 icon for Aurora

;---------------------
; Start of TI-83 PROGRAM
;---------------------

prog_start:
    call    _runIndicOff            ; Kill that annoying busy indicator
    call    _GRBUFCLR               ; Wipe the graph memory
    res     statsvalid,(iy+statflags)
    set     textwrite,(iy+SGRFLAGS) ; Send text to the graph memory

    ld      hl,TitleStr             ; Get the address of the string
    set     textEraseBelow,(iy+textflags)
    set     textInverse,(iy+textflags)
    ld      bc,$0011                ; Move to top of screen
    call    ShowStr                 ; Display the title string

    res     textInverse,(iy+textflags)
    ld      bc,$0712                ; Move down a line
    call    ShowStr                 ; Display the credit string

    ld      bc,$0E01                ; Move the cursor
    call    ShowStr                 ; Display the status string

    ld      bc,$1405                ; Move the cursor
    call    ShowStr                 ; Display the trigger mode string

    ld      bc,$1A05                ; Move down a line
    call    ShowStr                 ; Display the delay string

    ld      bc,$2201                ; Move the cursor
    call    ShowStr                 ; Display the Ch.1 string

    ld      bc,$3001                ; Move down a line
    call    ShowStr                 ; Display the Ch.2 string

    ld      bc,$0E22                ; Move up and across
    call    ShowStr                 ; Display the Idle string

    call    UpKeyHandler             ; Set menu item to zero

MainLoop:
    ld      hl,StrTrigMan           ; Offset to the first mode string
    ld      a,(TRIG_MODE)           ; Get the trigger mode in a

ChkTrigIdx:
    dec     a                       ; We're one string forward
    or      a                       ; If a equals zero..
    jr      z,FoundModeStr          ; ..we've found the string
    push    af                      ; Save that for later

    xor     a                       ; Find the end of a string
    ld      c,a                     ; Make sure we've got headroom
    cpir                            ; Search for the NULL character

    pop     af                      ; Get the mode index back
    jr      ChkTrigIdx              ; See if we're at the right string

FoundModeStr:
    ld      bc,$1422                ; Move the cursor
    call    ShowStr                 ; Display the string

    ld      hl,(SAMP_DLY)           ; Display sampling delay
    ld      h,0                     ; Get the delay into hl

    ld      de,$1A2A                ; Display the ones digit
    call    ShowOneDigit
    ld      de,$1A26                ; Display the tens digit
    call    ShowOneDigit
    ld      de,$1A22                ; Display the hundreds digit
    call    ShowOneDigit

    call    _GRBUFCPY_V             ; Update the screen

    ei                              ; We need the interrupts
KeyLoop:
    halt                            ; Save power
    call    $4014                   ; Check keypad
    or      a                       ; See if a key is pressed
    jr      z,KeyLoop               ; Loop if it isn't
    res     onInterrupt,(iy+onFlags)

    cp      $0F                     ; User hit CLEAR
    ret     z                       ; Exit

    cp      $09                     ; User says go!
    call    z,Acquire

    dec     a                       ; cp    G_DOWN
    call    z,DownKeyHandler

    dec     a                       ; cp    G_LEFT
    call    z,LeftKeyHandler

    dec     a                       ; cp    G_RIGHT
    call    z,RightKeyHandler

    dec     a                       ; cp    G_UP
    call    z,UpKeyHandler

    jr      MainLoop                ; Loop

;---------------------
; SUB for running an acquisition
;---------------------

Acquire:
    ld      bc,$1401                ; Set cursor location
    call    KillMenuMarker          ; Erase upper marker
    ld      bc,$1A01                ; Set cursor location
    call    KillMenuMarker          ; Erase lower marker

    ld      bc,$0E22                ; Tell the user what's going on
    ld      hl,StrStatusArmed       ; Get the string offset
    call    ShowStr                 ; Display it

    call    _GRBUFCPY_V             ; Update the screen

    di                              ; Maximize speed (no interrupts)
    ld      a,(SAMP_DLY)            ; Get delay in e
    ld      e,a
    inc     e                       ; Fix delay (0 = fastest)

    ld      bc,(192 << 8) + BPORT   ; Twice the number of samples in b
    ld      hl,ACQ_BUF              ; Buffer to store samples in

    ld      a,(TRIG_MODE)           ; Check trigger mode
    dec     a                       ; Manual trigger starts now
    jr      z,AcqLoop

    ld      d,Ch1Mask               ; Check for Ch.1 Modes
    dec     a
    jr      z,TrigRLpA
    dec     a
    jr      z,TrigFLpA

    ld      d,Ch2Mask               ; Check for Ch.2 Modes
    dec     a
    jr      z,TrigRLpA

TrigFLpA:                           ; Wait for a one
    in      a,(BPORT)
    and     d
    jr      z,TrigFLpA
TrigFLpB:                           ; Wait for a zero
    in      a,(BPORT)
    and     d
    jr      nz,TrigFLpB
    jr      AcqLoop                 ; Acquire

TrigRLpA:                           ; Wait for a zero
    in      a,(BPORT)
    and     d
    jr      nz,TrigRLpA
TrigRLpB:                           ; Wait for a one
    in      a,(BPORT)
    and     d
    jr      z,TrigRLpB

AcqLoop:                            ; This is where it happens
    ini                             ; Get a byte
    ld      d,b                     ; Save loop counter
    ld      b,e                     ; Get the delay byte into b
AcqDlyLoop:
    djnz    AcqDlyLoop              ; Generate the delay
    ld      b,d                     ; Restore loop counter
    djnz    AcqLoop                 ; Loop

    ld      bc,$0E22                ; Tell the user what's going on
    ld      hl,StrStatusBusy        ; Get the string offset
    call    ShowStr                 ; Display it

    call    _GRBUFCPY_V             ; Update the screen

    ld      ix,ACQ_BUF              ; Initialise the data counter
    ld      de,$0000                ; Initialise the screen counter
    ld      b,12                    ; Initialise loop counter
SampOutLoop:
    push    bc                      ; Save loop counter
    push    ix                      ; Save data counter

    ld      b,8                     ; Eight samples to the byte
    ld      c,%10000000             ; For lighting pixels
    xor     a                       ; Start with a blank byte
Ch1OutLoop:
    ld      h,(ix+$00)              ; Get a sample from buffer
    bit     Ch1Bit,h                ; See if Ch.1 was high
    jr      z,Ch1LoopNow            ; Loop if it wasn't
    or      c                       ; Light a pixel if it was
Ch1LoopNow:
    srl     c                       ; Move along one pixel
    inc     ix                      ; Move along one sample
    djnz    Ch1OutLoop              ; Loop

    ld      hl,PLOTSSCREEN+$01EC    ; Light the 1 line
    add     hl,de
    ld      (hl),a
    cpl                             ; Complement the pixels
    ld      hl,PLOTSSCREEN+$0228    ; Light the 0 line
    add     hl,de
    ld      (hl),a

    pop     ix                      ; Restore data counter
    ld      b,8                     ; Eight samples to the byte
    ld      c,%10000000             ; For lighting pixels
    xor     a                       ; Start with a blank byte
Ch2OutLoop:
    ld      h,(ix+$00)              ; Get a sample from buffer
    bit     Ch2Bit,h                ; See if Ch.1 was high
    jr      z,Ch2LoopNow            ; Loop if it wasn't
    or      c                       ; Light a pixel if it was
Ch2LoopNow:
    srl     c                       ; Move along one pixel
    inc     ix                      ; Move along one sample
    djnz    Ch2OutLoop              ; Loop

    ld      hl,PLOTSSCREEN+$0294    ; Light the 1 line
    add     hl,de
    ld      (hl),a
    cpl                             ; Complement the pixels
    ld      hl,PLOTSSCREEN+$02D0    ; Light the 0 line
    add     hl,de
    ld      (hl),a

    pop     bc                      ; Restore loop counter
    inc     de                      ; Increment screen counter
    djnz    SampOutLoop             ; Loop

    ld      bc,$0E22                ; Tell the user what's going on
    ld      hl,StrStatusIdle        ; Get the string offset
    call    ShowStr                 ; Display it

    ei                              ; We'll want the interrupts later
                                    ; Fall through to up key handler

;---------------------
; SUB for handling the up arrow key
;---------------------

UpKeyHandler:
    ld      bc,$1A01                ; Set cursor location
    call    KillMenuMarker          ; Erase old marker
    xor     a                       ; Set the menu item
    ld      (MENU_ITEM),a

    ld      de,$1401                ; Move cursor
    jr      NewMenuSel              ; Re-use code

;---------------------
; SUB for handling the down arrow key
;---------------------

DownKeyHandler:
    ld      a,1                     ; Set the menu item
    ld      (MENU_ITEM),a

    ld      bc,$1401                ; Set cursor location
    call    KillMenuMarker          ; Erase old marker

    ld      de,$1A01                ; Move cursor
NewMenuSel:
    ld      a,$05                   ; Display new marker
    call    ShowMenuChar

    xor     a
    ret                             ; Return

;---------------------
; SUB for handling the right arrow key
;---------------------

RightKeyHandler:
    ld      a,(MENU_ITEM)           ; See which item we're on
    or      a
    jr      z,IncTrigMode           ; Item zero is the trigger
    ld      hl,SAMP_DLY             ; Increment the sampling delay
    inc     (hl)
    xor     a
    ret                             ; Return

;---------------------
; SUB for handling the left arrow key
;---------------------

LeftKeyHandler:
    ld      a,(MENU_ITEM)           ; See which item we're on
    or      a
    jr      z,DecTrigMode           ; Item zero is the trigger
    ld      hl,SAMP_DLY             ; Decrement sampling delay
    dec     (hl)
    xor     a
    ret                             ; Return

;---------------------
; SUB for incrementing the trigger mode
;---------------------

IncTrigMode:
    ld      a,(TRIG_MODE)           ; Get the current mode in a
    inc     a                       ; Increment it
    cp      TRIG_2F+1               ; Return if it's Ch.2 Neg.
    jr      NewTrigMode             ; Re-use code

;---------------------
; SUB for decrementing the trigger mode
;---------------------

DecTrigMode:
    ld      a,(TRIG_MODE)           ; Get the current mode in a
    dec     a                       ; Decrement it
    or      a                       ; Return if it's zero
NewTrigMode:
    ret     z
    ld      (TRIG_MODE),a           ; Store new mode
    xor     a
    ret                             ; Return

;---------------------
; SUB for hiding the menu marker
;---------------------

KillMenuMarker:
    ld      hl,BlankStr             ; Get a few spaces cued

;---------------------
; SUB for showing the ASCIIZ string at offset hl at location (c,b)
;---------------------

ShowStr:
    ld      (PENCOL),bc             ; Set the cursor location
    jp      _vputs                  ; Display the string

;---------------------
; SUB for displaying a decimal digit of hl
;---------------------

ShowOneDigit:
    ld      a,10                    ; This is what we divide by
    call    _divHLbyA               ; Do the division
    add     a,'0'                   ; Convert to a character
ShowMenuChar:
    ld      (PENCOL),de             ; Move the cursor
    jp      _vputmap                ; Display it

;---------------------
; Program DATA
;---------------------

TRIG_MODE:                          ; Trigger mode
    .db 1
SAMP_DLY:                           ; Delay between samples
    .db 128

TitleStr:
    .db " Logic Analyser 1.3",0
    .db "by Vasantha Crabb",0

StatusStr:                          ; Strings on the main screen
    .db "Status:",0
TrigModeStr:
    .db "Trigger:",0
DelayStr:
    .db "Delay:",0
Ch1Str:
    .db "Channel 1 (Tip)",0
Ch2Str:
    .db "Channel 2 (Ring)",0

StrStatusIdle:                      ; Status display strings
    .db "Idle"
    .db "                    "
    .db "                   ",0
StrStatusArmed:
    .db "Trigger Armed",0
StrStatusBusy:
    .db "Processing Data",0

StrTrigMan:                         ; Trigger mode strings
    .db "Manual"
    .db "                         "
BlankStr:
    .db "   ",0
StrTrig1R:
    .db "Channel 1 Rising     ",0
StrTrig1F:
    .db "Channel 1 Falling",0
StrTrig2R:
    .db "Channel 2 Rising     ",0
StrTrig2F:
    .db "Channel 2 Falling",0

Icon:                               ; Aurora icon
    .db %11111110
    .db %10000010
    .db %10011110
    .db %10010010
    .db %11110010
    .db %10000010
    .db %11111110
    .db %00000000

.end
